interactions.curse.minor.bad_luck=function(idx)
    local tbl={}
    tbl=split_to_lines(tbl,[[
    [IS_TRIGGER_STRING_SECOND:have]
    [IS_TRIGGER_STRING_THIRD:has]
    [IS_TRIGGER_STRING:been cursed with a week's bad luck!]
[I_TARGET:A:CREATURE]
    [IT_LOCATION:CONTEXT_CREATURE]
    [IT_MANUAL_INPUT:victim]
[I_EFFECT:ADD_SYNDROME]
    [IE_TARGET:A]
    [IE_IMMEDIATE]
    [SYNDROME]
        [SYN_CONCENTRATION_ADDED:1000:0]//just in case
        [CE_SKILL_ROLL_ADJUST:PERC:0:PERC_ON:20:START:0:PEAK:0:END:8400:ABRUPT]//one week
        ]])
    return {raws=tbl,weight=1}
end

interactions.curse.medium.become_beast=function(idx)
    local tbl={}
    tbl=split_to_lines(tbl,[[
        [IS_TRIGGER_STRING_SECOND:have]
        [IS_TRIGGER_STRING_THIRD:has]
        [IS_TRIGGER_STRING:been cursed, for one week, to know life as a beast!]
    [I_TARGET:A:CREATURE]
        [IT_LOCATION:CONTEXT_CREATURE]
        [IT_MANUAL_INPUT:victim]
    [I_EFFECT:ADD_SYNDROME]
        [IE_TARGET:A]
        [IE_IMMEDIATE]
        [SYNDROME]
            [SYN_CONCENTRATION_ADDED:1000:0]//just in case
            [CE_BODY_TRANSFORMATION:START:0:PEAK:0:END:8400:ABRUPT]//one week
                [CE:CREATURE_CASTE_FLAG:NATURAL_ANIMAL]
                [CE:FORBIDDEN_CREATURE_FLAG:SMALL_RACE]
                [CE:FORBIDDEN_CREATURE_CASTE_FLAG:LARGE_PREDATOR]
                [CE:FORBIDDEN_CREATURE_CASTE_FLAG:CANNOT_BREATHE_AIR]
                [CE:FORBIDDEN_CREATURE_CASTE_FLAG:IMMOBILE_LAND]
                [CE:FORBIDDEN_CREATURE_CASTE_FLAG:CAN_LEARN]
                [CE:FORBIDDEN_CREATURE_CASTE_FLAG:MEGABEAST]
                [CE:FORBIDDEN_CREATURE_CASTE_FLAG:SEMIMEGABEAST]
                [CE:FORBIDDEN_CREATURE_CASTE_FLAG:TITAN]
                [CE:FORBIDDEN_CREATURE_CASTE_FLAG:DEMON]
                [CE:FORBIDDEN_CREATURE_CASTE_FLAG:UNIQUE_DEMON]
                [CE:FORBIDDEN_CREATURE_CASTE_FLAG:SUPERNATURAL]
                [CE:HAVE_FAST_EFFORTLESS_GAIT_SPEED:1000]
            [CE_ADD_TAG:NO_AGING:NO_EAT:NO_DRINK:START:0:PEAK:0:END:8400:ABRUPT]//one week
]])
    return {raws=tbl,weight=1}
end

interactions.curse.major.vampire=function(idx,tok)
    return {raws={
    "[IS_HIST_STRING_1: cursed ]",
    "[IS_HIST_STRING_2: to prowl the night in search of blood]",
    "[IS_TRIGGER_STRING_SECOND:have]",
    "[IS_TRIGGER_STRING_THIRD:has]",
    "[IS_TRIGGER_STRING:been cursed to prowl the night in search of blood!]",
"[I_SOURCE:INGESTION]",
    "[IS_HIST_STRING_1: consumed the tainted blood of ]",
    "[IS_HIST_STRING_2: and was cursed]",
"[I_TARGET:A:CREATURE]",
    "[IT_LOCATION:CONTEXT_CREATURE]",
    "[IT_REQUIRES:CAN_LEARN]",
    "[IT_REQUIRES:HAS_BLOOD]",
    "[IT_FORBIDDEN:NOT_LIVING]",
    "[IT_FORBIDDEN:SUPERNATURAL]",
    "[IT_CANNOT_HAVE_SYNDROME_CLASS:WERECURSE]",
    "[IT_CANNOT_HAVE_SYNDROME_CLASS:VAMPCURSE]",
    "[IT_CANNOT_HAVE_SYNDROME_CLASS:DISTURBANCE_CURSE]",
    "[IT_CANNOT_HAVE_SYNDROME_CLASS:RAISED_UNDEAD]",
    "[IT_CANNOT_HAVE_SYNDROME_CLASS:RAISED_GHOST]",
    "[IT_CANNOT_HAVE_SYNDROME_CLASS:GHOUL]",
"[I_EFFECT:ADD_SYNDROME]",
    "[IE_TARGET:A]",
    "[IE_IMMEDIATE]",
    "[IE_ARENA_NAME:Vampire]",
    "[SYNDROME]",
    --SOME STANDARD UNDEAD PROPERTIES, BUT NOT ALL OF THEM
        "[SYN_CLASS:VAMPCURSE]",
        "[SYN_CONCENTRATION_ADDED:1000:0]",--just in case
        "[CE_ADD_TAG:BLOODSUCKER:NO_AGING:STERILE:NOT_LIVING:NOEXERT:NOPAIN:NOBREATHE:NOSTUN:NONAUSEA:NO_DIZZINESS:NO_FEVERS:PARALYZEIMMUNE:NO_EAT:NO_DRINK:NO_SLEEP:NO_PHYS_ATT_GAIN:NO_PHYS_ATT_RUST:START:0:ABRUPT]",
        "[CE_PHYS_ATT_CHANGE:STRENGTH:200:0:AGILITY:200:0:TOUGHNESS:200:0:START:0:ABRUPT]",
        "[CE_MATERIAL_FORCE_MULTIPLIER:MAT_MULT:NONE:NONE:1:2:ABRUPT]",
        "[CE_BODY_MAT_INTERACTION:MAT_TOKEN:RESERVED_BLOOD:START:0:ABRUPT]",
        "[CE:INTERACTION:"..tok.."]",
            "[CE:SYNDROME_TAG:SYN_INGESTED]",
                    "[CE:SYNDROME_TAG:SYN_INJECTED]",
        "[CE_DISPLAY_TILE:TILE:165:4:0:0:START:0:CAN_BE_HIDDEN:ABRUPT]",
        "[CE_DISPLAY_NAME:NAME:vampire:vampires:vampiric:START:0:CAN_BE_HIDDEN:ABRUPT]",
        "[CE_BP_APPEARANCE_MODIFIER:START:0:BP:BY_CATEGORY:TOOTH:APPEARANCE_MODIFIER:LENGTH:150:ABRUPT]",
            "[CE:COUNTER_TRIGGER:DRINKING_BLOOD:1:NONE:REQUIRED]",
        "[CE_SENSE_CREATURE_CLASS:START:0:CLASS:GENERAL_POISON:15:4:0:1:ABRUPT]"
    },weight=1}
end